#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "evenbetter-rll.h"

#define noDUMP_FILE_NAME "dump.ebdp"

static void
die (char *why)
{
  fprintf (stderr, "%s\n", why);
  exit (1);
}

#define MAX_SIZE 65536

#define M 16

int
main (int argc, char **argv)
{
  FILE *fi = stdin;
  FILE *fo = stdout;
  char buf[256];
  int xs, ys;
  int xsb;
  int depth;
  EvenBetterParams params;
  EvenBetterCtx *ctx;
  int lut[256], i;
  int *luts[M];
  int rs_lut[256];
  int *rs_luts[M];
  int strengths[] = { 0 };
  int even_c1_scale[] = { 1 };

  unsigned char *ibuf, *obuf;
  int y;
  int nbytes;

  fgets (buf, sizeof(buf), fi);
  if (buf[0] != 'P' || buf[1] != '5')
    die ("Need pgmraw image on input");

  xs = ys = 0;
  do
    fgets (buf, sizeof(buf), fi);
  while (buf[0] == '#');
  sscanf (buf, "%d %d", &xs, &ys);
  if (xs <= 0 || ys <= 0 || xs > MAX_SIZE || ys > MAX_SIZE)
    die ("Input image size out of range");

  do
    fgets (buf, sizeof(buf), fi);
  while (buf[0] == '#');
  sscanf (buf, "%d", &depth);
  if (depth != 255)
    die ("Only works with depth=255 images");

  fprintf (fo, "P5\n%d %d\n255\n", xs, ys);

  ibuf = malloc (xs);
  xsb = xs;
  obuf = eb_malloc_aligned (xsb + 16, 16);

  /* This sets up a simple gamma lookup table. */
  if (argc == 2)
    {
      FILE *lutf = fopen (argv[1], "r");
      for (i = 0; i < 256; i++)
	fscanf (lutf, "%d", &lut[i]);
      fclose (lutf);
    }
  else
    {
      for (i = 0; i < 256; i++)
#if 1
	lut[i] = (1 << 24) * (pow (i / 255.0, 1.0));
#else
	lut[i] = (1 << 24) * (0.88 + 0.02 * i / 255.0);
#endif
    }

  for (i = 0; i < 256; i++)
    rs_lut[i] = 2 << 16;

#ifdef DUMP_FILE_NAME
  params.dump_file = fopen (DUMP_FILE_NAME, "wb");
#else
  params.dump_file = NULL;
#endif
  params.dump_level = EB_DUMP_ALL;

  params.source_width = xs;
  params.dest_width = xs;
  params.n_planes = 1;
  params.levels = 2;
  luts[0] = lut;
  params.luts = luts;
  params.strengths = strengths;
  params.aspect = 1;
  params.rbscale = 0;
  params.rand_scale = 5;
  params.even_c1_scale = even_c1_scale;
  params.do_shadows = 0;
  params.gamma = 0.0;
  rs_luts[0] = rs_lut;
  params.rand_scale_luts = rs_luts;
  params.rand_scale_luts = NULL;
  ctx = even_better_new (&params);
  for (y = 0; y < ys; y++)
    {
      int x;
      const ET_SrcPixel *ibufs[1];
      uchar *obufs[1];

      nbytes = fread (ibuf, 1, xs, fi);
      if (nbytes != xs)
        die ("Error reading image (file truncated?)");
      ibufs[0] = ibuf;
      obufs[0] = obuf;
      even_better_line (ctx, &obuf, (const ET_SrcPixel *const *)&ibufs);
      for (x = 0; x < xs; x++)
	{
	  char byte = obuf[x];
	  char obuf2[1];

	  obuf2[0] = 255 - byte * 255 / (params.levels - 1);
	  nbytes = 1;
	  if (xs - x < 1)
	    nbytes = xs - x;
	  nbytes = fwrite (obuf2, 1, nbytes, fo);
	}
    }

  even_better_free (ctx);
  eb_free_aligned(obuf);
  free (ibuf);

  return 0;
}

